from django.contrib.auth.backends import ModelBackend
from django.contrib.auth.models import User

from ..backends import RateLimitMixin, RateLimitModelBackend


class TestBackend(RateLimitModelBackend):
    minutes = 10
    requests = 50

    def key(self, request, dt):
        """Derives the cache key from the submitted username too."""
        return '%s%s-%s-%s' % (
            self.cache_prefix,
            request.META.get('REMOTE_ADDR', ''),
            request.POST['username'],
            dt.strftime('%Y%m%d%H%M'),
        )


class CustomBackend(ModelBackend):
    def authenticate(self, token=None, secret=None):
        try:
            user = User.objects.get(username=token)
            if user.check_password(secret):
                return user
        except User.DoesNotExist:
            return None


class TestCustomBackend(RateLimitMixin, CustomBackend):
    """Rate-limited backend with token/secret instead of username/password"""
    username_key = 'token'


class TestCustomBrokenBackend(RateLimitMixin, CustomBackend):
    """Rate-limited backend with token/secret instead of username/password"""



from django.db import models

try:
    from django.contrib.auth.models import AbstractBaseUser, BaseUserManager
    from django.utils import timezone
except ImportError:
    pass
else:
    class UserManager(BaseUserManager):
        def create_user(self, email, password=None, **extra_fields):
            now = timezone.now()
            if not email:
                raise ValueError('The email must be set.')
            email = UserManager.normalize_email(email)
            user = self.model(email=email, last_login=now, date_joined=now,
                              **extra_fields)
            user.set_password(password)
            user.save(using=self._db)
            return user

        def create_superuser(self, email, password, **extra_fields):
            user = self.create_user(email, password, **extra_fields)
            user.is_staff = True
            user.is_active = True
            user.is_superuser = True
            user.save(using=self._db, update_fields=['is_staff', 'is_active',
                                                     'is_superuser'])
            return user

    class User(AbstractBaseUser):
        """A user with email as identifier"""
        USERNAME_FIELD = 'email'
        REQUIRED_FIELDS = []
        email = models.EmailField(max_length=255, unique=True, db_index=True)
        is_staff = models.BooleanField(default=False)
        is_active = models.BooleanField(default=False)
        is_superuser = models.BooleanField(default=False)
        date_joined = models.DateTimeField(default=timezone.now)

        objects = UserManager()



from django.conf.urls import include, patterns, url

from .. import admin
from ..views import login
from .forms import CustomAuthForm

admin.autodiscover()


urlpatterns = patterns(
    '',
    url(r'^login/$', login,
        {'template_name': 'admin/login.html'}, name='login'),
    url(r'^custom_login/$', login,
        {'template_name': 'custom_login.html',
         'authentication_form': CustomAuthForm},
        name='custom_login'),
    url(r'^admin/', include(admin.site.urls)),
)



#-*- coding: utf-8 -*-
import warnings

import django

from django.conf import settings
from django.contrib.auth.models import User
from django.core.cache import cache
from django.core.urlresolvers import reverse
from django.test import TestCase
from django.test.utils import override_settings
from django.utils.unittest import skipIf

try:
    from django.contrib.auth import get_user_model
except ImportError:
    pass


class RateLimitTests(TestCase):
    urls = 'ratelimitbackend.tests.urls'

    def setUp(self):
        cache.clear()

    def assertRateLimited(self, response):
        self.assertContains(response, 'Too many failed login attempts',
                            status_code=403)

    @override_settings(AUTH_USER_MODEL='tests.User')
    @skipIf(django.VERSION[:2] < (1, 5), "Swappable user not available")
    def test_ratelimit_login_attempt_swapped_user(self):
        url = reverse('login')
        response = self.client.get(url)
        self.assertContains(response, 'username')
        self.assertContains(response, 'password')

        wrong_data = {
            'username': u'hï',
            'password': 'suspicious attempt',
        }
        # 30 failing attempts are allowed
        for iteration in range(30):
            response = self.client.post(url, wrong_data)
            self.assertContains(response, 'username')

        response = self.client.post(url, wrong_data)
        self.assertRateLimited(response)

        # IP is rate-limited; even valid login attempts are blocked.
        get_user_model().objects.create_user('foo@bar.com', 'pass')
        response = self.client.post(url, {'username': 'foo',
                                          'password': 'pass'})
        self.assertRateLimited(response)

    def test_ratelimit_login_attempt(self):
        url = reverse('login')
        response = self.client.get(url)
        self.assertContains(response, 'username')
        self.assertContains(response, 'password')

        wrong_data = {
            'username': u'hï',
            'password': 'suspicious attempt',
        }
        # 30 failing attempts are allowed
        for iteration in range(30):
            response = self.client.post(url, wrong_data)
            self.assertContains(response, 'username')

        response = self.client.post(url, wrong_data)
        self.assertRateLimited(response)

        # IP is rate-limited; even valid login attempts are blocked.
        User.objects.create_user('foo', 'foo@bar.com', 'pass')
        response = self.client.post(url, {'username': 'foo',
                                          'password': 'pass'})
        self.assertRateLimited(response)

    @override_settings(AUTH_USER_MODEL='tests.User')
    @skipIf(django.VERSION[:2] < (1, 5), "Swappable user not available")
    def test_ratelimit_login_attempt_swapped(self):
        url = reverse('login')
        response = self.client.get(url)
        self.assertContains(response, 'username')
        self.assertContains(response, 'password')

        wrong_data = {
            'username': u'hï',
            'password': 'suspicious attempt',
        }
        # 30 failing attempts are allowed
        for iteration in range(30):
            response = self.client.post(url, wrong_data)
            self.assertContains(response, 'username')

        response = self.client.post(url, wrong_data)
        self.assertRateLimited(response)

        # IP is rate-limited; even valid login attempts are blocked.
        get_user_model().objects.create_user('foo@bar.com', 'pass')
        response = self.client.post(url, {'username': 'foo',
                                          'password': 'pass'})
        self.assertRateLimited(response)

    def test_ratelimit_admin_logins(self):
        url = reverse('admin:index')
        response = self.client.get(url)
        self.assertContains(response, 'username')
        wrong_data = {
            'username': u'hî',
            'password': 'suspicious attempt',
        }
        # 30 failing attempts are allowed
        for iteration in range(30):
            response = self.client.post(url, wrong_data)
            self.assertContains(response, 'username')

        response = self.client.post(url, wrong_data)
        self.assertRateLimited(response)

    def test_django_registry(self):
        user = User.objects.create_user('username', 'foo@bar.com', 'pass')
        user.is_staff = True
        user.is_superuser = True
        user.save()
        with warnings.catch_warnings(record=True) as w:
            self.client.login(request=None, username='username',
                              password='pass')
            self.assertEqual(len(w), 1)
        url = reverse('admin:index')
        response = self.client.get(url)
        self.assertContains(response, 'in the Auth application')
        self.assertContains(response, '"/admin/auth/user/add/"')

    def test_custom_ratelimit_logic(self):
        """Custom backend behaviour"""
        url = reverse('login')
        old_backends = settings.AUTHENTICATION_BACKENDS
        settings.AUTHENTICATION_BACKENDS = (
            'ratelimitbackend.tests.backends.TestBackend',
        )

        wrong_data = {
            'username': u'ùser1',
            'password': 'suspicious attempt',
        }
        # 50 failing attempts are allowed
        for iteration in range(50):
            response = self.client.post(url, wrong_data)
            self.assertContains(response, 'username')

        # Attempts for this username are blocked
        response = self.client.post(url, wrong_data)
        self.assertRateLimited(response)

        # Further attempts with another username are allowed
        wrong_data['username'] = 'user2'
        response = self.client.post(url, wrong_data)
        self.assertContains(response, 'username')

        settings.AUTHENTICATION_BACKENDS = old_backends

    @override_settings(AUTHENTICATION_BACKENDS=(
        'ratelimitbackend.tests.backends.TestCustomBackend',))
    def test_custom_backend(self):
        """Backend with custom authentication method"""
        url = reverse('custom_login')
        response = self.client.get(url)
        self.assertContains(response, 'token')
        self.assertContains(response, 'secret')

        wrong_data = {
            'token': u'hï',
            'secret': 'suspicious attempt',
        }
        # 30 failed attempts are allowed
        for iteration in range(30):
            response = self.client.post(url, wrong_data)
        self.assertContains(response, 'secret')

        response = self.client.post(url, wrong_data)
        self.assertRateLimited(response)

        # IP is rate-limited; even valid login attempts are blocked.
        User.objects.create_user('foo', 'foo@bar.com', 'pass')
        response = self.client.post(url, {'token': 'foo',
                                          'secret': 'pass'})
        self.assertRateLimited(response)

    @override_settings(AUTHENTICATION_BACKENDS=(
        'ratelimitbackend.tests.backends.TestCustomBrokenBackend',))
    def test_custom_backend_no_username_key(self):
        """Custom backend with missing username_key"""
        url = reverse('custom_login')
        wrong_data = {
            'token': u'hï',
            'secret': 'suspicious attempt',
        }
        self.assertRaises(KeyError, self.client.post, url, wrong_data)



from django.forms import Form, ValidationError, CharField, PasswordInput
from django.contrib.auth import authenticate


class CustomAuthForm(Form):
    token = CharField(max_length=30)
    secret = CharField(widget=PasswordInput)

    def __init__(self, request=None, *args, **kwargs):
        self.request = request
        self.user_cache = None
        super(CustomAuthForm, self).__init__(*args, **kwargs)

    def clean(self):
        token = self.cleaned_data.get('token')
        secret = self.cleaned_data.get('secret')
        if token and secret:
            self.user_cache = authenticate(token=token,
                                           secret=secret,
                                           request=self.request)
            if self.user_cache is None:
                raise ValidationError("Invalid")
            elif not self.user_cache.is_active:
                raise ValidationError("Inactive")
        return self.cleaned_data

    def get_user(self):
        return self.user_cache
